# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'main.ui',
# licensing of 'main.ui' applies.
#
# Created: Fri Mar 27 12:26:29 2020
#      by: pyside2-uic  running on PySide2 5.13.2
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets

class Ui_MondoMainWindow(object):
    def setupUi(self, MondoMainWindow):
        MondoMainWindow.setObjectName("MondoMainWindow")
        MondoMainWindow.resize(617, 351)
        self.centralwidget = QtWidgets.QWidget(MondoMainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.groupBox = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.psdButton = QtWidgets.QPushButton(self.groupBox)
        self.psdButton.setMinimumSize(QtCore.QSize(175, 0))
        self.psdButton.setObjectName("psdButton")
        self.verticalLayout.addWidget(self.psdButton)
        self.singleChannelPsdButton = QtWidgets.QPushButton(self.groupBox)
        self.singleChannelPsdButton.setObjectName("singleChannelPsdButton")
        self.verticalLayout.addWidget(self.singleChannelPsdButton)
        self.singleSlicePsdButton = QtWidgets.QPushButton(self.groupBox)
        self.singleSlicePsdButton.setObjectName("singleSlicePsdButton")
        self.verticalLayout.addWidget(self.singleSlicePsdButton)
        self.overlaidPsdButton = QtWidgets.QPushButton(self.groupBox)
        self.overlaidPsdButton.setMinimumSize(QtCore.QSize(175, 0))
        self.overlaidPsdButton.setObjectName("overlaidPsdButton")
        self.verticalLayout.addWidget(self.overlaidPsdButton)
        spacerItem = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.gridLayout.addWidget(self.groupBox, 0, 2, 1, 1)
        self.groupBox_4 = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox_4.setObjectName("groupBox_4")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.groupBox_4)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.csvButton = QtWidgets.QPushButton(self.groupBox_4)
        self.csvButton.setMinimumSize(QtCore.QSize(175, 0))
        self.csvButton.setObjectName("csvButton")
        self.verticalLayout_4.addWidget(self.csvButton)
        self.csvDownsampledButton = QtWidgets.QPushButton(self.groupBox_4)
        self.csvDownsampledButton.setMinimumSize(QtCore.QSize(175, 0))
        self.csvDownsampledButton.setObjectName("csvDownsampledButton")
        self.verticalLayout_4.addWidget(self.csvDownsampledButton)
        spacerItem1 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem1)
        self.gridLayout.addWidget(self.groupBox_4, 1, 0, 1, 1)
        self.groupBox_2 = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.spectrogramButton = QtWidgets.QPushButton(self.groupBox_2)
        self.spectrogramButton.setMinimumSize(QtCore.QSize(175, 0))
        self.spectrogramButton.setObjectName("spectrogramButton")
        self.verticalLayout_2.addWidget(self.spectrogramButton)
        spacerItem2 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem2)
        self.gridLayout.addWidget(self.groupBox_2, 0, 1, 1, 1)
        self.groupBox_3 = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox_3.setObjectName("groupBox_3")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_3)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.timeButton = QtWidgets.QPushButton(self.groupBox_3)
        self.timeButton.setMinimumSize(QtCore.QSize(175, 0))
        self.timeButton.setObjectName("timeButton")
        self.verticalLayout_3.addWidget(self.timeButton)
        self.synchronousTimeButton = QtWidgets.QPushButton(self.groupBox_3)
        self.synchronousTimeButton.setMinimumSize(QtCore.QSize(175, 0))
        self.synchronousTimeButton.setObjectName("synchronousTimeButton")
        self.verticalLayout_3.addWidget(self.synchronousTimeButton)
        self.overlaidTimeButton = QtWidgets.QPushButton(self.groupBox_3)
        self.overlaidTimeButton.setMinimumSize(QtCore.QSize(175, 0))
        self.overlaidTimeButton.setObjectName("overlaidTimeButton")
        self.verticalLayout_3.addWidget(self.overlaidTimeButton)
        spacerItem3 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem3)
        self.gridLayout.addWidget(self.groupBox_3, 0, 0, 1, 1)
        self.groupBox_5 = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox_5.setObjectName("groupBox_5")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.groupBox_5)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.fileInformationButton = QtWidgets.QPushButton(self.groupBox_5)
        self.fileInformationButton.setObjectName("fileInformationButton")
        self.verticalLayout_6.addWidget(self.fileInformationButton)
        self.viewSettingsButton = QtWidgets.QPushButton(self.groupBox_5)
        self.viewSettingsButton.setObjectName("viewSettingsButton")
        self.verticalLayout_6.addWidget(self.viewSettingsButton)
        self.rawExportButton = QtWidgets.QPushButton(self.groupBox_5)
        self.rawExportButton.setObjectName("rawExportButton")
        self.verticalLayout_6.addWidget(self.rawExportButton)
        self.splitFileButton = QtWidgets.QPushButton(self.groupBox_5)
        self.splitFileButton.setObjectName("splitFileButton")
        self.verticalLayout_6.addWidget(self.splitFileButton)
        spacerItem4 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_6.addItem(spacerItem4)
        self.gridLayout.addWidget(self.groupBox_5, 1, 1, 1, 1)
        self.verticalLayout_5.addLayout(self.gridLayout)
        spacerItem5 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_5.addItem(spacerItem5)
        self.verticalLayout_5.setStretch(1, 1)
        MondoMainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MondoMainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 617, 21))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        self.menuCheckForUpdates = QtWidgets.QMenu(self.menuHelp)
        self.menuCheckForUpdates.setObjectName("menuCheckForUpdates")
        self.menuSettings = QtWidgets.QMenu(self.menubar)
        self.menuSettings.setObjectName("menuSettings")
        MondoMainWindow.setMenuBar(self.menubar)
        self.actionAbout = QtWidgets.QAction(MondoMainWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.actionExit = QtWidgets.QAction(MondoMainWindow)
        self.actionExit.setObjectName("actionExit")
        self.actionAboutLibraries = QtWidgets.QAction(MondoMainWindow)
        self.actionAboutLibraries.setObjectName("actionAboutLibraries")
        self.actionUpdateLatestStable = QtWidgets.QAction(MondoMainWindow)
        self.actionUpdateLatestStable.setObjectName("actionUpdateLatestStable")
        self.actionUpdateSpecificBranch = QtWidgets.QAction(MondoMainWindow)
        self.actionUpdateSpecificBranch.setObjectName("actionUpdateSpecificBranch")
        self.actionUpdateSpecificCommit = QtWidgets.QAction(MondoMainWindow)
        self.actionUpdateSpecificCommit.setObjectName("actionUpdateSpecificCommit")
        self.actionUpdateCurrentBranch = QtWidgets.QAction(MondoMainWindow)
        self.actionUpdateCurrentBranch.setObjectName("actionUpdateCurrentBranch")
        self.actionPreferences = QtWidgets.QAction(MondoMainWindow)
        self.actionPreferences.setObjectName("actionPreferences")
        self.menuFile.addAction(self.actionExit)
        self.menuCheckForUpdates.addAction(self.actionUpdateLatestStable)
        self.menuCheckForUpdates.addAction(self.actionUpdateCurrentBranch)
        self.menuCheckForUpdates.addAction(self.actionUpdateSpecificBranch)
        self.menuCheckForUpdates.addAction(self.actionUpdateSpecificCommit)
        self.menuHelp.addAction(self.menuCheckForUpdates.menuAction())
        self.menuHelp.addSeparator()
        self.menuHelp.addAction(self.actionAbout)
        self.menuHelp.addAction(self.actionAboutLibraries)
        self.menuSettings.addAction(self.actionPreferences)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuSettings.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MondoMainWindow)
        QtCore.QObject.connect(self.actionExit, QtCore.SIGNAL("triggered()"), MondoMainWindow.close)
        QtCore.QMetaObject.connectSlotsByName(MondoMainWindow)

    def retranslateUi(self, MondoMainWindow):
        MondoMainWindow.setWindowTitle(QtWidgets.QApplication.translate("MondoMainWindow", "Mondo", None, -1))
        self.groupBox.setTitle(QtWidgets.QApplication.translate("MondoMainWindow", "Frequency Analysis", None, -1))
        self.psdButton.setText(QtWidgets.QApplication.translate("MondoMainWindow", "Power Spectral Density (PSD)", None, -1))
        self.singleChannelPsdButton.setToolTip(QtWidgets.QApplication.translate("MondoMainWindow", "Overlaid PSD streamlined for plotting a single channel across multiple time slices", None, -1))
        self.singleChannelPsdButton.setText(QtWidgets.QApplication.translate("MondoMainWindow", "Single Channel Overlaid PSD", None, -1))
        self.singleSlicePsdButton.setToolTip(QtWidgets.QApplication.translate("MondoMainWindow", "Overlaid PSD streamlined for plotting multiple channels in a single time slice", None, -1))
        self.singleSlicePsdButton.setText(QtWidgets.QApplication.translate("MondoMainWindow", "Single Time Slice Overlaid PSD", None, -1))
        self.overlaidPsdButton.setToolTip(QtWidgets.QApplication.translate("MondoMainWindow", "Ovelaid PSD for plotting arbitrary channels and time slices ", None, -1))
        self.overlaidPsdButton.setText(QtWidgets.QApplication.translate("MondoMainWindow", "Overlaid PSD", None, -1))
        self.groupBox_4.setTitle(QtWidgets.QApplication.translate("MondoMainWindow", "Raw Data", None, -1))
        self.csvButton.setText(QtWidgets.QApplication.translate("MondoMainWindow", "CSV Output", None, -1))
        self.csvDownsampledButton.setText(QtWidgets.QApplication.translate("MondoMainWindow", "CSV Downsampled Output", None, -1))
        self.groupBox_2.setTitle(QtWidgets.QApplication.translate("MondoMainWindow", "Time/Frequency Analysis", None, -1))
        self.spectrogramButton.setText(QtWidgets.QApplication.translate("MondoMainWindow", "Spectrogram", None, -1))
        self.groupBox_3.setTitle(QtWidgets.QApplication.translate("MondoMainWindow", "Time Analysis", None, -1))
        self.timeButton.setText(QtWidgets.QApplication.translate("MondoMainWindow", "Time Data", None, -1))
        self.synchronousTimeButton.setText(QtWidgets.QApplication.translate("MondoMainWindow", "Synchronous Time Data", None, -1))
        self.overlaidTimeButton.setText(QtWidgets.QApplication.translate("MondoMainWindow", "Overlaid Time Data", None, -1))
        self.groupBox_5.setTitle(QtWidgets.QApplication.translate("MondoMainWindow", "File Manipulation", None, -1))
        self.fileInformationButton.setText(QtWidgets.QApplication.translate("MondoMainWindow", "File Information", None, -1))
        self.viewSettingsButton.setText(QtWidgets.QApplication.translate("MondoMainWindow", "View Settings", None, -1))
        self.rawExportButton.setText(QtWidgets.QApplication.translate("MondoMainWindow", "Advanced Raw Export", None, -1))
        self.splitFileButton.setText(QtWidgets.QApplication.translate("MondoMainWindow", "Rend File Asunder", None, -1))
        self.menuFile.setTitle(QtWidgets.QApplication.translate("MondoMainWindow", "File", None, -1))
        self.menuHelp.setTitle(QtWidgets.QApplication.translate("MondoMainWindow", "Help", None, -1))
        self.menuCheckForUpdates.setTitle(QtWidgets.QApplication.translate("MondoMainWindow", "Check for Updates", None, -1))
        self.menuSettings.setTitle(QtWidgets.QApplication.translate("MondoMainWindow", "Settings", None, -1))
        self.actionAbout.setText(QtWidgets.QApplication.translate("MondoMainWindow", "About Mondo...", None, -1))
        self.actionAbout.setToolTip(QtWidgets.QApplication.translate("MondoMainWindow", "About Mondo...", None, -1))
        self.actionExit.setText(QtWidgets.QApplication.translate("MondoMainWindow", "Exit", None, -1))
        self.actionAboutLibraries.setText(QtWidgets.QApplication.translate("MondoMainWindow", "About Libraries...", None, -1))
        self.actionAboutLibraries.setToolTip(QtWidgets.QApplication.translate("MondoMainWindow", "About Libraries...", None, -1))
        self.actionUpdateLatestStable.setText(QtWidgets.QApplication.translate("MondoMainWindow", "Latest Stable", None, -1))
        self.actionUpdateSpecificBranch.setText(QtWidgets.QApplication.translate("MondoMainWindow", "Specific Branch...", None, -1))
        self.actionUpdateSpecificCommit.setText(QtWidgets.QApplication.translate("MondoMainWindow", "Specific Commit..", None, -1))
        self.actionUpdateCurrentBranch.setText(QtWidgets.QApplication.translate("MondoMainWindow", "Latest branch", None, -1))
        self.actionPreferences.setText(QtWidgets.QApplication.translate("MondoMainWindow", "Preferences...", None, -1))
        self.actionPreferences.setToolTip(QtWidgets.QApplication.translate("MondoMainWindow", "Preferences...", None, -1))

