# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'datetime_subset.ui',
# licensing of 'datetime_subset.ui' applies.
#
# Created: Mon Mar  2 17:09:12 2020
#      by: pyside2-uic  running on PySide2 5.13.2
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets

class Ui_DateTimeSubsetDialog(object):
    def setupUi(self, DateTimeSubsetDialog):
        DateTimeSubsetDialog.setObjectName("DateTimeSubsetDialog")
        DateTimeSubsetDialog.resize(283, 284)
        self.gridLayout = QtWidgets.QGridLayout(DateTimeSubsetDialog)
        self.gridLayout.setObjectName("gridLayout")
        spacerItem = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 4, 0, 1, 3)
        self.useAllData = QtWidgets.QRadioButton(DateTimeSubsetDialog)
        self.useAllData.setChecked(True)
        self.useAllData.setObjectName("useAllData")
        self.gridLayout.addWidget(self.useAllData, 0, 0, 1, 3)
        self.buttonBox = QtWidgets.QDialogButtonBox(DateTimeSubsetDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 5, 0, 1, 3)
        self.formLayout_2 = QtWidgets.QFormLayout()
        self.formLayout_2.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout_2.setObjectName("formLayout_2")
        self.startLabel = QtWidgets.QLabel(DateTimeSubsetDialog)
        self.startLabel.setEnabled(False)
        self.startLabel.setObjectName("startLabel")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.startLabel)
        self.startDateTime = QtWidgets.QDateTimeEdit(DateTimeSubsetDialog)
        self.startDateTime.setEnabled(False)
        self.startDateTime.setWrapping(True)
        self.startDateTime.setObjectName("startDateTime")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.startDateTime)
        self.endLabel = QtWidgets.QLabel(DateTimeSubsetDialog)
        self.endLabel.setEnabled(False)
        self.endLabel.setObjectName("endLabel")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.endLabel)
        self.endDateTime = QtWidgets.QDateTimeEdit(DateTimeSubsetDialog)
        self.endDateTime.setEnabled(False)
        self.endDateTime.setWrapping(True)
        self.endDateTime.setObjectName("endDateTime")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.endDateTime)
        self.durationLabel = QtWidgets.QLabel(DateTimeSubsetDialog)
        self.durationLabel.setEnabled(False)
        self.durationLabel.setObjectName("durationLabel")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.durationLabel)
        self.label = QtWidgets.QLabel(DateTimeSubsetDialog)
        self.label.setEnabled(False)
        self.label.setObjectName("label")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label)
        self.label_2 = QtWidgets.QLabel(DateTimeSubsetDialog)
        self.label_2.setEnabled(False)
        self.label_2.setObjectName("label_2")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.startSeconds = QtWidgets.QSpinBox(DateTimeSubsetDialog)
        self.startSeconds.setEnabled(False)
        self.startSeconds.setObjectName("startSeconds")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.startSeconds)
        self.endSeconds = QtWidgets.QSpinBox(DateTimeSubsetDialog)
        self.endSeconds.setEnabled(False)
        self.endSeconds.setObjectName("endSeconds")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.endSeconds)
        self.duration = QtWidgets.QSpinBox(DateTimeSubsetDialog)
        self.duration.setEnabled(False)
        self.duration.setObjectName("duration")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.duration)
        self.gridLayout.addLayout(self.formLayout_2, 3, 1, 1, 2)
        spacerItem1 = QtWidgets.QSpacerItem(13, 13, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem1, 3, 0, 1, 1)
        self.useSubset = QtWidgets.QRadioButton(DateTimeSubsetDialog)
        self.useSubset.setObjectName("useSubset")
        self.gridLayout.addWidget(self.useSubset, 2, 0, 1, 3)
        self.formLayout_4 = QtWidgets.QFormLayout()
        self.formLayout_4.setObjectName("formLayout_4")
        self.allStartLabel = QtWidgets.QLabel(DateTimeSubsetDialog)
        self.allStartLabel.setObjectName("allStartLabel")
        self.formLayout_4.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.allStartLabel)
        self.allStart = QtWidgets.QLabel(DateTimeSubsetDialog)
        self.allStart.setObjectName("allStart")
        self.formLayout_4.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.allStart)
        self.allEndLabel = QtWidgets.QLabel(DateTimeSubsetDialog)
        self.allEndLabel.setObjectName("allEndLabel")
        self.formLayout_4.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.allEndLabel)
        self.allEnd = QtWidgets.QLabel(DateTimeSubsetDialog)
        self.allEnd.setObjectName("allEnd")
        self.formLayout_4.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.allEnd)
        self.allDurationLabel = QtWidgets.QLabel(DateTimeSubsetDialog)
        self.allDurationLabel.setObjectName("allDurationLabel")
        self.formLayout_4.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.allDurationLabel)
        self.allDuration = QtWidgets.QLabel(DateTimeSubsetDialog)
        self.allDuration.setObjectName("allDuration")
        self.formLayout_4.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.allDuration)
        self.gridLayout.addLayout(self.formLayout_4, 1, 1, 1, 2)

        self.retranslateUi(DateTimeSubsetDialog)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), DateTimeSubsetDialog.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("rejected()"), DateTimeSubsetDialog.reject)
        QtCore.QObject.connect(self.useSubset, QtCore.SIGNAL("toggled(bool)"), self.startLabel.setEnabled)
        QtCore.QObject.connect(self.useSubset, QtCore.SIGNAL("toggled(bool)"), self.startDateTime.setEnabled)
        QtCore.QObject.connect(self.useSubset, QtCore.SIGNAL("toggled(bool)"), self.endLabel.setEnabled)
        QtCore.QObject.connect(self.useSubset, QtCore.SIGNAL("toggled(bool)"), self.endDateTime.setEnabled)
        QtCore.QObject.connect(self.useSubset, QtCore.SIGNAL("toggled(bool)"), self.durationLabel.setEnabled)
        QtCore.QObject.connect(self.useAllData, QtCore.SIGNAL("toggled(bool)"), self.allStartLabel.setEnabled)
        QtCore.QObject.connect(self.useAllData, QtCore.SIGNAL("toggled(bool)"), self.allStart.setEnabled)
        QtCore.QObject.connect(self.useAllData, QtCore.SIGNAL("toggled(bool)"), self.allEndLabel.setEnabled)
        QtCore.QObject.connect(self.useAllData, QtCore.SIGNAL("toggled(bool)"), self.allEnd.setEnabled)
        QtCore.QObject.connect(self.useAllData, QtCore.SIGNAL("toggled(bool)"), self.allDurationLabel.setEnabled)
        QtCore.QObject.connect(self.useAllData, QtCore.SIGNAL("toggled(bool)"), self.allDuration.setEnabled)
        QtCore.QObject.connect(self.useSubset, QtCore.SIGNAL("toggled(bool)"), self.label.setEnabled)
        QtCore.QObject.connect(self.useSubset, QtCore.SIGNAL("toggled(bool)"), self.label_2.setEnabled)
        QtCore.QObject.connect(self.useSubset, QtCore.SIGNAL("toggled(bool)"), self.startSeconds.setEnabled)
        QtCore.QObject.connect(self.useSubset, QtCore.SIGNAL("toggled(bool)"), self.endSeconds.setEnabled)
        QtCore.QObject.connect(self.useSubset, QtCore.SIGNAL("toggled(bool)"), self.duration.setEnabled)
        QtCore.QMetaObject.connectSlotsByName(DateTimeSubsetDialog)
        DateTimeSubsetDialog.setTabOrder(self.useAllData, self.useSubset)
        DateTimeSubsetDialog.setTabOrder(self.useSubset, self.startDateTime)
        DateTimeSubsetDialog.setTabOrder(self.startDateTime, self.endDateTime)
        DateTimeSubsetDialog.setTabOrder(self.endDateTime, self.buttonBox)

    def retranslateUi(self, DateTimeSubsetDialog):
        DateTimeSubsetDialog.setWindowTitle(QtWidgets.QApplication.translate("DateTimeSubsetDialog", "Select Data Range", None, -1))
        self.useAllData.setText(QtWidgets.QApplication.translate("DateTimeSubsetDialog", "Use all data", None, -1))
        self.startLabel.setText(QtWidgets.QApplication.translate("DateTimeSubsetDialog", "Start Time (UTC):", None, -1))
        self.startDateTime.setDisplayFormat(QtWidgets.QApplication.translate("DateTimeSubsetDialog", "M/d/yyyy h:mm:ss AP", None, -1))
        self.endLabel.setText(QtWidgets.QApplication.translate("DateTimeSubsetDialog", "End Time (UTC):", None, -1))
        self.endDateTime.setDisplayFormat(QtWidgets.QApplication.translate("DateTimeSubsetDialog", "M/d/yyyy h:mm:ss AP", None, -1))
        self.durationLabel.setText(QtWidgets.QApplication.translate("DateTimeSubsetDialog", "Duration:", None, -1))
        self.label.setText(QtWidgets.QApplication.translate("DateTimeSubsetDialog", "Relative Start Time", None, -1))
        self.label_2.setText(QtWidgets.QApplication.translate("DateTimeSubsetDialog", "Relative End Time", None, -1))
        self.startSeconds.setSuffix(QtWidgets.QApplication.translate("DateTimeSubsetDialog", " s", None, -1))
        self.endSeconds.setSuffix(QtWidgets.QApplication.translate("DateTimeSubsetDialog", " s", None, -1))
        self.duration.setSuffix(QtWidgets.QApplication.translate("DateTimeSubsetDialog", " s", None, -1))
        self.useSubset.setText(QtWidgets.QApplication.translate("DateTimeSubsetDialog", "Use data within range", None, -1))
        self.allStartLabel.setText(QtWidgets.QApplication.translate("DateTimeSubsetDialog", "Start Time (UTC):", None, -1))
        self.allStart.setText(QtWidgets.QApplication.translate("DateTimeSubsetDialog", "1/1/2000 12:00:00 AM", None, -1))
        self.allEndLabel.setText(QtWidgets.QApplication.translate("DateTimeSubsetDialog", "End Time (UTC):", None, -1))
        self.allEnd.setText(QtWidgets.QApplication.translate("DateTimeSubsetDialog", "1/1/2000 12:00:00 AM", None, -1))
        self.allDurationLabel.setText(QtWidgets.QApplication.translate("DateTimeSubsetDialog", "Duration:", None, -1))
        self.allDuration.setText(QtWidgets.QApplication.translate("DateTimeSubsetDialog", "0 s", None, -1))

