# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'psd_options.ui',
# licensing of 'psd_options.ui' applies.
#
# Created: Tue Apr  2 19:36:21 2019
#      by: pyside2-uic  running on PySide2 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets

class Ui_PSDOptionsWidget(object):
    def setupUi(self, PSDOptionsWidget):
        PSDOptionsWidget.setObjectName("PSDOptionsWidget")
        PSDOptionsWidget.resize(208, 191)
        self.formLayout = QtWidgets.QFormLayout(PSDOptionsWidget)
        self.formLayout.setContentsMargins(0, 0, 0, 0)
        self.formLayout.setObjectName("formLayout")
        self.fftPointsLabel = QtWidgets.QLabel(PSDOptionsWidget)
        self.fftPointsLabel.setObjectName("fftPointsLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.fftPointsLabel)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem)
        self.fftPoints = QtWidgets.QComboBox(PSDOptionsWidget)
        self.fftPoints.setObjectName("fftPoints")
        self.horizontalLayout_4.addWidget(self.fftPoints)
        self.formLayout.setLayout(0, QtWidgets.QFormLayout.FieldRole, self.horizontalLayout_4)
        self.durationLabel = QtWidgets.QLabel(PSDOptionsWidget)
        self.durationLabel.setObjectName("durationLabel")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.durationLabel)
        self.duration = QtWidgets.QLabel(PSDOptionsWidget)
        self.duration.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.duration.setObjectName("duration")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.duration)
        self.resolutionLabel = QtWidgets.QLabel(PSDOptionsWidget)
        self.resolutionLabel.setObjectName("resolutionLabel")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.resolutionLabel)
        self.resolution = QtWidgets.QLabel(PSDOptionsWidget)
        self.resolution.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.resolution.setObjectName("resolution")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.resolution)
        self.overlapPercentLabel = QtWidgets.QLabel(PSDOptionsWidget)
        self.overlapPercentLabel.setObjectName("overlapPercentLabel")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.overlapPercentLabel)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.overlapPercent = QtWidgets.QDoubleSpinBox(PSDOptionsWidget)
        self.overlapPercent.setMinimumSize(QtCore.QSize(75, 0))
        self.overlapPercent.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.overlapPercent.setDecimals(0)
        self.overlapPercent.setMaximum(95.0)
        self.overlapPercent.setSingleStep(5.0)
        self.overlapPercent.setProperty("value", 75.0)
        self.overlapPercent.setObjectName("overlapPercent")
        self.horizontalLayout_3.addWidget(self.overlapPercent)
        self.formLayout.setLayout(3, QtWidgets.QFormLayout.FieldRole, self.horizontalLayout_3)
        self.overlapPointsLabel = QtWidgets.QLabel(PSDOptionsWidget)
        self.overlapPointsLabel.setObjectName("overlapPointsLabel")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.overlapPointsLabel)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem2)
        self.overlapPoints = QtWidgets.QSpinBox(PSDOptionsWidget)
        self.overlapPoints.setMinimumSize(QtCore.QSize(75, 0))
        self.overlapPoints.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.overlapPoints.setObjectName("overlapPoints")
        self.horizontalLayout_5.addWidget(self.overlapPoints)
        self.formLayout.setLayout(4, QtWidgets.QFormLayout.FieldRole, self.horizontalLayout_5)
        self.windowCountLabel = QtWidgets.QLabel(PSDOptionsWidget)
        self.windowCountLabel.setObjectName("windowCountLabel")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.windowCountLabel)
        self.windowCount = QtWidgets.QLabel(PSDOptionsWidget)
        self.windowCount.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.windowCount.setObjectName("windowCount")
        self.formLayout.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.windowCount)
        self.windowFunctionLabel = QtWidgets.QLabel(PSDOptionsWidget)
        self.windowFunctionLabel.setObjectName("windowFunctionLabel")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.windowFunctionLabel)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.windowFunction = QtWidgets.QComboBox(PSDOptionsWidget)
        self.windowFunction.setObjectName("windowFunction")
        self.horizontalLayout_2.addWidget(self.windowFunction)
        self.formLayout.setLayout(6, QtWidgets.QFormLayout.FieldRole, self.horizontalLayout_2)
        self.detrendMethodLabel = QtWidgets.QLabel(PSDOptionsWidget)
        self.detrendMethodLabel.setObjectName("detrendMethodLabel")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.LabelRole, self.detrendMethodLabel)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem4)
        self.detrendMethod = QtWidgets.QComboBox(PSDOptionsWidget)
        self.detrendMethod.setObjectName("detrendMethod")
        self.horizontalLayout.addWidget(self.detrendMethod)
        self.formLayout.setLayout(7, QtWidgets.QFormLayout.FieldRole, self.horizontalLayout)

        self.retranslateUi(PSDOptionsWidget)
        QtCore.QMetaObject.connectSlotsByName(PSDOptionsWidget)

    def retranslateUi(self, PSDOptionsWidget):
        PSDOptionsWidget.setWindowTitle(QtWidgets.QApplication.translate("PSDOptionsWidget", "PSD Options", None, -1))
        self.fftPointsLabel.setText(QtWidgets.QApplication.translate("PSDOptionsWidget", "FFT Window Size", None, -1))
        self.durationLabel.setText(QtWidgets.QApplication.translate("PSDOptionsWidget", "Window Duration", None, -1))
        self.duration.setText(QtWidgets.QApplication.translate("PSDOptionsWidget", "600 s", None, -1))
        self.resolutionLabel.setText(QtWidgets.QApplication.translate("PSDOptionsWidget", "Frequency Resolution", None, -1))
        self.resolution.setText(QtWidgets.QApplication.translate("PSDOptionsWidget", "0.1 Hz", None, -1))
        self.overlapPercentLabel.setText(QtWidgets.QApplication.translate("PSDOptionsWidget", "Window Overlap Percent", None, -1))
        self.overlapPercent.setSuffix(QtWidgets.QApplication.translate("PSDOptionsWidget", " %", None, -1))
        self.overlapPointsLabel.setText(QtWidgets.QApplication.translate("PSDOptionsWidget", "Window Overlap Points", None, -1))
        self.windowCountLabel.setText(QtWidgets.QApplication.translate("PSDOptionsWidget", "Number of Windows", None, -1))
        self.windowCount.setText(QtWidgets.QApplication.translate("PSDOptionsWidget", "1", None, -1))
        self.windowFunctionLabel.setText(QtWidgets.QApplication.translate("PSDOptionsWidget", "Window Function", None, -1))
        self.detrendMethodLabel.setText(QtWidgets.QApplication.translate("PSDOptionsWidget", "Detrend Method", None, -1))

